/**
 * The object that creates a distributed search with Apache Hadoop and runs
 * it. Also contains the main function needed to run Multisearch from the
 * command line.
 * <p>
 * Last Updated: 31 July 2008
 * <p>
 *
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch;

import org.apache.hadoop.mapred.jobcontrol.*;
import java.io.File;
import java.io.IOException;
import java.io.DataInputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.StringTokenizer;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;
import org.apache.hadoop.mapred.lib.IdentityReducer;
import edu.arsc.multisearch.select.*;

public class Multisearch extends Configured implements Tool {

    /** The query for the multisearch. */
    private static String multisearchSuperQuery;

    /** The query number associated with this multisearch. */
    private static int multisearchSuperQueryNumber = 0;

    /** The label associated with this multisearch. */
    private static String multisearchSuperLabel = "ANY";

    /** Merge function.*/  
    private static String multisearchSuperMerge = "edu.arsc.multisearch.merge.NaiveMergeSet";

    /** The restriction/selection function associated with this multisearch */
    private static String multisearchSuperSelect = "";

    /** The output class for Hadoop. */
    private static String outputClass = "edu.arsc.multisearch.TRECOutputFormat";

    /** Folder with the input files. */
    private static String inputFiles = "allinput/";

    //Number of services 
    private static int limit = 10;

    /** Counters via Hadoop.*/
    static enum Counters {INPUT_SERVICES, OUTPUT_RESULTS};

    /** Output file for results.*/
    private String outputLocation;

    /** Parses queries. */
    private static QueryParser parser;

    //some other stuff
    private String outputFileBase = "hadoopOutput/";    
    private String finoutfile = outputFileBase + "/finaloutput.out";
    private String jarFile = "Multisearch.jar";
    private File inputFile;
    private String[] qArgs;

    private java.io.PrintWriter web = null;
  

    /**
     * Runs a Multisearch and prints the results.
     */
    public static void main(String args[]) {

	if(args.length == 0 || args.length == 1) {

	    System.out.println("Appropriate format for running Multisearch 3.0:"); 
	    System.out.println("-r restriction -m full.merge.set.class.name -q query terms");
	    System.out.println("or");
	    System.out.println("-r restricton -m full.merge.set.class.name -f filename with batch queries");
	    System.out.println("-o OutputType (class)");
	    System.out.println("-i input files (directory)");
	    System.out.println("Batch query format expected is query#:query and a newline between each");
	    System.out.println("");
	    System.out.println("If no MergeClass is specified, NaiveMergeSet will be used.");
	    System.out.println("If no Restriction/Selection class is specified, none will be used.");

	} else {

	    parser = new QueryParser();

	    String query = ""; 
	    String restrict = "";
	    String merge = "";
	    String filename = "";

	    for(int q = 0; q < args.length; q++) {

		if(args[q].equals("-q")) {

		    for(int a = q+1; a < args.length; a++) {

			if(args[a].equals("-r") || args[a].equals("-m")) {

			    break;

			} else {

			    query += " " + args[a];

			}

		    }


		    query = parser.parse(query);			

		} else if(args[q].equals("-r") && (q+1) < args.length) {

		    restrict = args[q+1];
		    q++;

		} else if(args[q].equals("-m") && (q+1) < args.length) {

		    merge = args[q+1];
		    q++;

		} else if(args[q].equals("-f") && (q+1) < args.length) {
		    
		    filename = args[q+1];
		    q++;

		} else if(args[q].equals("-i") && (q+1) < args.length) {

		    inputFiles = args[q+1];
		    q++;

		} else if(args[q].equals("-o") && (q+1) < args.length) {

		    outputClass = args[q+1];
		    q++;

		} else if(args[q].equals("-l") && (q+1) < args.length) {

		    limit = java.lang.Integer.parseInt(args[q+1]);
		    q++;

		}
	      

	    } //end for-args-length

	    if(query.equals("") && filename.equals("")) {

		System.out.println("Error: No query or filename provided!");

	    } else {

		//set valueable variables
		if(!merge.equals("")) {
		    
		    multisearchSuperMerge = merge;
		    
		}
		
		if(!restrict.equals("")) {
		    
		    multisearchSuperSelect= restrict;
		    
		}
		    


		if(!filename.equals("")) {

		    File inputFileIn = new File(filename);
		    Multisearch search = new Multisearch(inputFileIn);
		    search.cleanUp();

		} else { 

		    multisearchSuperQuery = query;
		    Multisearch search = new Multisearch(query);

		    long start = System.currentTimeMillis();
		    search.run(search.getArgs());
		    long end = System.currentTimeMillis();
		    long elapsed = end - start;
		    float seconds = elapsed/1000F;
		    float minutes = elapsed/(60*1000F);

		    System.out.println("Search took " + seconds + " seconds, or " + minutes + " minutes.");

		}


		//clean up input
		if(!multisearchSuperSelect.equals("") && !multisearchSuperSelect.equals("allinput/")) {

		    File inFile = new File(inputFiles);
		    cleanInput(inFile);

		}



	    }


	}

	
    }

    /**
     * Create a Multisearch object with only one query.
     *
     * @param String query
     */
    public Multisearch(String query) {

	//Arguments
	multisearchSuperQuery = query;
	qArgs = new String[1];
	qArgs[0] = query;

    }

    /**
     * Create a Multisearch object with a file full of queries.
     *
     * @param File file
     */
    public Multisearch(File file) {

	try {
	
	    //Open the need File Inputs
	    FileInputStream fis = new FileInputStream(file);
	    BufferedInputStream bis = new BufferedInputStream(fis);
	    DataInputStream in = new DataInputStream(bis);

	    //Extract the queries and run this
	    while(in.available() != 0) {

		String line = in.readLine();
		StringTokenizer st = new StringTokenizer(line, ":");
		String qNum = st.nextToken();
		String query = st.nextToken();

		multisearchSuperQuery = query;
		multisearchSuperQueryNumber = java.lang.Integer.parseInt(qNum);

		String[] qArgs = new String[3];
		qArgs[0] = query;
		qArgs[1] = qNum;
		qArgs[2] = outputFileBase + "/" + qNum;

		//if this has been set, use it
		if(!multisearchSuperSelect.equals("")) {
		    
		    System.out.println("The restriction class " + multisearchSuperSelect + " is being run.");
		    
		    //Figure out which restriction class
		    Class rMaker = Class.forName(multisearchSuperSelect);
		    
		    //create it
		    Select select = (Select)rMaker.newInstance();
		    
		    inputFiles = select.restrict(query, limit);
		    
		}

		long start = System.currentTimeMillis();
		run(qArgs);
		long end = System.currentTimeMillis();
		
		long elapsed = end - start;
		float seconds = elapsed/1000F;
		float minutes = elapsed/(60*1000F);
		
		System.out.println("Search took " + seconds + " seconds, or " + minutes + " minutes.");

	    }

	    //close the file inputs
	    in.close();
	    bis.close();
	    fis.close();

	} catch(java.io.FileNotFoundException fnfe) {

	    System.out.println(fnfe.toString());

	} catch(java.io.IOException ioe) {

	    System.out.println(ioe.toString());

	} catch(Exception e) {

	    e.printStackTrace();

	}


    }


    /**
     * Run function
     */
    public int run(String args[]) {

	try {

	    //if this has been set, use it
	    if(!multisearchSuperSelect.equals("")) {
	    
		//Figure out which restriction class
		Class rMaker = Class.forName(multisearchSuperSelect);
		
		//create it
		Select select = (Select)rMaker.newInstance();
		
		inputFiles = select.restrict(args[0], limit);

	    } 
	    
	    outputLocation = outputFileBase;
	    
	    if(args.length > 2) {
		
		outputLocation = args[2];
		
	    }

	    //JobConf conf = new JobConf(getConf(), Multisearch.class);
	    JobConf conf = new JobConf(new Configuration(), Multisearch.class);
	    conf.setJobName("multisearch");
	    conf.setJar(jarFile);
	    
	    //Paths
	    ServiceInputFormat.setInputPaths(conf, new Path(inputFiles));
	    ResultSetOutputFormat.setOutputPath(conf, new Path(outputLocation));
	    
	    //Service and Merge classes
	    conf.setMapperClass(edu.arsc.multisearch.ServiceMap.class);
	    conf.setCombinerClass(edu.arsc.multisearch.MergeReduce.class);
	    conf.setReducerClass(edu.arsc.multisearch.MergeReduce.class);
	    
	    //Mapper outputs
	    conf.setMapOutputKeyClass(org.apache.hadoop.io.Text.class);
	    conf.setMapOutputValueClass(edu.arsc.multisearch.ResultSetWritable.class);
	    
	    //Merge outputs
	    conf.setOutputKeyClass(org.apache.hadoop.io.Text.class);
	    conf.setOutputValueClass(edu.arsc.multisearch.ResultSetWritable.class);
	    
	    //input/output formats
	    Class outputC = Class.forName(outputClass);
	    conf.setOutputFormat(outputC);
	    conf.setInputFormat(edu.arsc.multisearch.ServiceInputFormat.class);
	    
		
	    JobClient.runJob(conf);
	    
	} catch(java.io.IOException ioe) {

	    if(web != null) {

		web.println("<p>@Multisearch.run(): " + ioe.toString() + "</p>");
		ioe.printStackTrace(web);	

	    }
    
	    System.err.println("@Multisearch.run(): " + ioe.toString());

	} catch(java.lang.Exception e) {

	    if(web != null) {

		web.println("<p>@Multisearch.run(): " + e.toString() + "</p>");
		e.printStackTrace(web);

	    }


	    System.err.println("@Multisearch.run(): " + e.toString());
	    e.printStackTrace();

	}

	System.out.println("complete.");

	return(1);

    }

    /**
     * remove this kylie
     */
    public void setPrint(java.io.PrintWriter web) {

	this.web = web;

    }


    /**
     * Return the query associated with this Multisearch object.
     *
     * @return String
     */
    public static String getQuery() {

	return(multisearchSuperQuery);

    }

    /**
     * @return String[]
     */
    public String[] getArgs() {

	return(qArgs);

    }

    /**
     * Return the label associated with this Multisearch object.
     *
     * @return String
     */
    public static String getLabel() {

	return(multisearchSuperLabel);

    }

    /**
     * Return the queryNumber associated with this Multisearch object.
     *
     * @return int
     */
    public static int getQueryNumber() {

	return(multisearchSuperQueryNumber);

    }

    /**
     * Return the merge class associated with this Multisearch object
     *
     * @return String
     */
    public static String getMergeClass() {

	return(multisearchSuperMerge);

    }

    /**
     * @param String
     */
    public void setMergeClass(String classSet) {

	multisearchSuperMerge = classSet;

    }

    /**
     * @param String
     */
    public void setRestriction(String classSet) {

	multisearchSuperSelect = classSet;

    }

    /**
     * Given multiple outputs during a batch run, this function 
     * combines all results into a single file in basepath called
     * final-results
     *
     */
    public void cleanUp() {

	File file = new File(outputFileBase);

	if(!file.exists() || !file.isDirectory()) {

	    System.out.println("File base " + outputFileBase + " is not a directory or does not exist.");

	} else {

	    try { 

		File outFile = new File(finoutfile);

		if(!outFile.exists()) {

		    outFile.createNewFile();

		}
		    
		FileWriter fw = new FileWriter(outFile);
		BufferedWriter out = new BufferedWriter(fw);
		
		//get all the output files
		File[] files = file.listFiles();
		
		for(int x = 0 ; x < files.length; x++) {
		    
		    if(files[x].isDirectory()) {
			
			File[] inFile = files[x].listFiles();
			
			for(int y = 0; y < inFile.length; y++) {

			    if(!inFile[y].isHidden()) {
			    
				//Open the input Stream
				FileInputStream fis = new FileInputStream(inFile[y]);
				BufferedInputStream bis = new BufferedInputStream(fis);
				DataInputStream in = new DataInputStream(bis);
				
				//Write to main output
				while(in.available() != 0) {
				    
				    out.write("\n" + in.readLine());
				    
				}
			    
				//Close input streams
				in.close();
				bis.close();
				fis.close();

			    } //end-if-file-is-hidden
			    
			}// end-for-files-in-output/#-directory
			
		    } //end if-files-in-output/#-directory
		    
		} // end-for-files-in-top-directory
		
		
		out.close();		

	    } catch(Exception e) {

		e.printStackTrace();

	    }
	    
	} //end if output directory is valid


    }


    /**
     * delete all
     */
    public static void cleanInput(File folder) {

        File[] lf = folder.listFiles();

        for(int i = 0; i < lf.length; i++) {

            if(lf[i].isDirectory()) {

                cleanInput(lf[i]);

            } else {

                lf[i].delete();

            }

        }

        folder.delete();


    }



    /**
     * @return String outputLocation
     */
    public String getOutputFile() {

	return(outputLocation);

    }
    

    /**
     * @return String inputFiles
     */
    public static String getInput() {

	return(inputFiles);

    }

    /**/
    public static void setOutputType(String ot) {

	outputClass = ot;

    }

    /**
     */
    public static void setInput(String in) {

	inputFiles = in;

    }

    /**
     * @param int 
     */
    public static void setLimit(int lim) {

	limit = lim;

    }

}
