/**
 * A class that reads the records provided by the generated service file
 * and makes them into useable InputSplits.
 *
 * @author Kylie McCormick
 * @date 22 July 2008
 * @project Multisearch
 */
package edu.arsc.multisearch;

import java.io.IOException;
import org.apache.hadoop.mapred.RecordReader;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileSplit;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.StringReader;
import java.io.BufferedInputStream;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.InputSource;

public class ServiceRecordReader implements RecordReader<Text, ServiceWritable> {

    /** Object that reads in Services via XML.*/
    private ServiceReader in = null;

    /**
     * A class that provides the reader from an input stream.
     */
    public static class ServiceReader {

	//note: may have to revert to deprecated dis-read-line if available() does not work
	private DataInputStream in;
	private BufferedInputStream bis;
	private InputStream inStream;
	
	/** 
	 * Create a new ServiceReader from the given stream
	 *
	 * @param InputStream in
	 * @throws java.io.IOException
	 */
	ServiceReader(InputStream inStream) throws java.io.IOException {

	    this.inStream = inStream;
	    bis = new BufferedInputStream(inStream);
	    in = new DataInputStream(bis);
	
	}

	/**
	 * Closes the underlying stream.
	 *
	 * @throws IOException
	 */
	public void close() throws IOException {

	    inStream.close();
	    bis.close();
	    in.close();

	}

	/**
	 * Returns the status of the input, eg: How close to end of file we are.
	 *
	 * @return int 
	 * @throws java.io.IOException
	 */
	public int getStatus() throws java.io.IOException {

	    return(in.available());

	}

	/**
	 * Reads in an XML chunk that defines a service. &lt;service&gt;&lt;/service&gt;
	 *
	 * @param ServiceWritable to update
	 * @throws java.io.IOException, javax.xml.parsers.ParserConfigurtionException, org.xml.sax.SAXException
	 */	 
	public void readService(ServiceWritable sw) 
	    throws java.io.IOException, javax.xml.parsers.ParserConfigurationException, org.xml.sax.SAXException{

	    String xmlstring = "";
	    boolean complete = false;

	    while(in.available() != 0 && !complete) {
		
		String input = in.readLine();
		xmlstring += input;
		    
		while(!input.equals("</service>")) {
		    
		    input = in.readLine();
		    xmlstring += input;
		    
		} //end-while-not-xml-service-end
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = factory.newDocumentBuilder();
		InputSource instream = new InputSource();
		instream.setCharacterStream(new StringReader(xmlstring));
		org.w3c.dom.Document document = db.parse(instream);
		
		NodeList uNL = document.getElementsByTagName("url");
		NodeList nNL = document.getElementsByTagName("name");
		NodeList cNL = document.getElementsByTagName("clientclass");
		
		Node urlNode = uNL.item(0);
		Node nameNode = nNL.item(0);
		Node clientNode = cNL.item(0);
		
		org.w3c.dom.Text urlText = (org.w3c.dom.Text)urlNode.getFirstChild();
		org.w3c.dom.Text nameText = (org.w3c.dom.Text)nameNode.getFirstChild();
		org.w3c.dom.Text clientText = (org.w3c.dom.Text)clientNode.getFirstChild();
		
		String URL = urlText.getNodeValue().trim();
		String name = nameText.getNodeValue().trim();
		String client = clientText.getNodeValue().trim();

		sw.setURL(URL);
		sw.setName(name);
		sw.setClientClass(client);

		xmlstring = "";
		complete = true;
	    }
	    
	}
    
    } //end-inner-class-declaration
	 
    /**
     * @param Configuration
     * @param FileSplit
     * @param String
     * @throws IOException
     */
    public ServiceRecordReader(Configuration job, FileSplit split) 
	throws java.io.IOException {

	try { 

	    final Path file = split.getPath();
	    FileSystem fs = file.getFileSystem(job);
	    FSDataInputStream fileIn = fs.open(split.getPath());
	    in = new ServiceReader(fileIn);	     	    

	} catch(java.lang.NullPointerException npe) {

	    System.out.println("ServiceRecordReader@ " + npe.toString());
	    npe.printStackTrace();

	}

    }


    /**
     * Returns number of bytes used by object. (Between 0 and 1 should be returned)
     *
     * @return float
     * @throws java.io.IOException
     */
    public float getProgress() throws java.io.IOException {

	float value = (float)in.getStatus();
	
	return(value);

    }

    /**
     * Close this split to future operations.
     */
    public void close() {

	try {

	    if(in != null) {

		in.close();

	    }

	} catch(java.io.IOException e) {

	    //error catching
	    System.out.println("@ServiceRecordReader: " + e.toString());
	}

    }

    /**
     * Returns current position in the input.
     *
     * @return long
     * @throws java.io.IOException
     */
    public long getPos() throws java.io.IOException {

	long value = (long)in.getStatus();

	return(value);

    }

    /**
     * Create a ServiceWritable object to be use as the value for this key, value pair
     *
     * @return ServiceWritable
     */
    public ServiceWritable createValue()  {

             
	try { 	

	    ServiceWritable sw = new ServiceWritable();
	    in.readService(sw);
	    return(sw);

	} catch(javax.xml.parsers.ParserConfigurationException pce) {
	    
	    //error printing
	    System.err.println("@ServiceRecordReader: " + pce.toString());
	    
	} catch(org.xml.sax.SAXException se) {
	    
	    //error printing
	    System.err.println("@ServiceRecordReader: " + se.toString());
	    
	} catch(java.io.IOException ioe) {

	    //error printing
	    System.err.println("@ServiceRecordReader: " + ioe.toString());

	}


	return(null);
    }

    /**
     * Generate an object for this to be used as a key.
     *
     * @return Text
     */
    public Text createKey() {

	String textKey = Multisearch.getQuery();
	Text text = new Text(textKey);
	return(text);

    }

    /**
     * Reads the next service into the object
     *
     * @param Text key
     * @param ServiceWritable service
     * @return boolean
     * @throws IOException
     */
    public synchronized boolean next(Text key, ServiceWritable service) 
				    throws IOException {

	    //check for end of stream
	    if(in.getStatus() <= 0) {

		return(false);

	    } else {

		//generate the service associated with this
		service = createValue();
		
		//generate the key associated with this
		key = createKey();	
		//key = new Text(Multisearch.getQuery());
	       
		return(true);				
	    }	    

    }

}
