/**
 * An object that can index files via Lucene.
 *
 * @author Kylie McCormick
 * @project Multisearch
 * @date 23 June 2008
 */
package edu.arsc.multisearch.backend.lucene;

import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

public class LuceneIndexer {

    /** The base URI for indexes. */
    public static String baseDir = "/home/mccormic/merge/indexes/";
    

    /** the base string for the index to be indexed */
    public static String index = "luc";

    /** the base string for the data files */
    public static String dataDirS = "/home/mccormic/merge/data/";

    /** Another base string */
    public static String baseData = "data/";

    public static void main(String args[]) throws Exception {
       
	if(args.length > 0) {

	    baseData = args[1];

	} 

	if(args.length > 1) {

	    index = args[0];

	}

	File indexDir = new File(baseDir, index);
	File dataDir = new File(dataDirS, baseData);


	int numIndexed = index(indexDir, dataDir);
	System.out.println("Indexed " + numIndexed + " with Lucene. Complete.");
    }


    /**
     * Opens the directory provided and starts to index the files.
     *
     * @param File indexDir            the directory to store the index
     * @param File dataDir             the directory that the data is in
     * @return int numIndexed          the number of files indexed
     */
    private static int index(File indexDir, File dataDir) throws IOException {

	int numIndexed = 0;

	if(!dataDir.exists() | !dataDir.isDirectory()) {

	    System.out.println("ERROR: The data director is not valid. Please check the directory, which is  " + dataDir.toString());

	} else {

	    IndexWriter writer = new IndexWriter(indexDir, new StandardAnalyzer(), true);
	    writer.setUseCompoundFile(false);
	    indexDirectory(writer, dataDir);
	    numIndexed = writer.docCount();
	    writer.optimize();
	    writer.close();


	}

	return(numIndexed);

    }

    /**
     * Recursive function that calls itself when it finds a directory
     * but calls the file searcher when it finds a file.
     *
     * @param IndexWriter writer           the declared writer for the index
     * @param File dir                     the directory in question
     */
    private static void indexDirectory(IndexWriter writer, File dir) throws IOException {

	File[] files = dir.listFiles();

	for(int i = 0; i < files.length; i++) {

	    File f = files[i];
	    	    
	    if(f.isDirectory()) {
		
		indexDirectory(writer, f);

	    } else {

		indexFile(writer, f);

	    }

	}
    }

    /**
     * Index a non-directory file, via Lucene
     *
     * @param IndexWriter writer      the declared writer for the index
     * @param File f                  the file to be indexed
     */
    private static void indexFile(IndexWriter writer, File f) throws IOException {

	if(f.isHidden() || !f.exists() || !f.canRead()) {

	    return;

	}

	System.out.println("Indexing " + f.getCanonicalPath());
	Document doc = new Document();

	doc.add(new Field("content", new FileReader(f)));
	doc.add(new Field("filename", f.getCanonicalPath(), Field.Store.YES, Field.Index.UN_TOKENIZED));
	

	writer.addDocument(doc);


    }


}
