/**
 * An object that represents a multisearch result set that matches
 * to other searches in Multisearch. Can be serialized with NetBeans.
 * <p>
 * Last Updated: 22 July 2008
 *
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch;

import java.io.DataInput;
import java.io.DataOutput;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.util.*;
import java.util.StringTokenizer;
import java.util.LinkedList;

public class ResultSetWritable implements Writable {

    /** Array of results, assumed in order. */
    private DocumentWritable[] results;

    /** Name of the server/backend from which these results came.*/
    private String name;

    /** Any other information the server/backend wants to be known.*/
    private String info;

    /** OrderedList for new documents in FinalSet.*/
    private OrderedList ranked;


    /**
     * @param DataInput
     */
    public void readFields(DataInput input) {

	try {


	    String line = input.readLine();
	    StringTokenizer st = new StringTokenizer(line, "=");

	    /**
	       Format, as in write() is the following:
	       Name:
	       Info:
	       Documents: (as many as there are)

		    lineInput += "=" + (i+1) + "=" + docs[i].getTitle() + "=" + 
			docs[i].getScore() + "=" + docs[i].getOldRank() + "=" + 
			docs[i].getOldScore()+ "=" + docs[i].getFilename();		

			=rank=title=score=oldrank=oldscore=filename
	     **/

	    //String nothing = st.nextToken();
	    String name = st.nextToken();
	    String info = st.nextToken();
	    LinkedList ll = new LinkedList();

	    //Document Collection
	    while(st.hasMoreTokens()){

		boolean complete = false;

		while(!complete && st.hasMoreTokens()) {
		    
		    String rank = st.nextToken();
		    String title = st.nextToken();
		    String score = st.nextToken();
		    String oldrank = st.nextToken();
		    String oldscore = st.nextToken();
		    String filename = st.nextToken();
		    String nameDoc = st.nextToken();

		    int rankI = Integer.parseInt(rank);
		    int oldrankI = Integer.parseInt(oldrank);
		    double scoreD = Double.parseDouble(score);
		    double oldscoreD = Double.parseDouble(oldscore);

		    DocumentWritable dw = new DocumentWritable(rankI, oldrankI, title, scoreD, oldscoreD, filename, nameDoc);
		    ll.add(dw);
		    
		    complete = true;

		}
		

	    }

	    DocumentWritable[] docs = new DocumentWritable[ll.size()];

	    for(int x = 0; x < ll.size(); x++) {

		docs[x] = (DocumentWritable)ll.get(x);

	    }
	    
	    this.info = info;
	    this.name = name;
	    this.results = docs;


	} catch(java.io.IOException ioe) {


	}

    }

    /**
     * @param DataOutput
     */
    public void write(DataOutput output) {

	try {

	    String lineInput = name + "=" + info;
	    DocumentWritable[] docs = results;

	    if(docs != null) {

		for(int i = 0; i < docs.length; i++) {
		    
		    String title = docs[i].getTitle();
		    double score = docs[i].getScore();
		    int oldrank = docs[i].getOldRank();
		    double oldscore = docs[i].getOldScore();
		    String filename = docs[i].getFilename();
		    String name = docs[i].getName();

		    if(title.equals("")) {
			
			title = "untitled";
		    }

		    if(filename.equals("")) {

			filename = "unknown";

		    }

		    if(name.equals("")) {

			name = "unknown";

		    }


		    
		    lineInput += "=" + (i+1) + "=" + title + "=" + score + "=" + oldrank + "=" + oldscore + "=" + filename + "=" + name;

		}

		
	    }

	    lineInput+= "\n";
	    output.writeBytes(lineInput);


	} catch(java.io.IOException ioe) {

	    

	}

    }

    /**
     * needed for Hadoop's Reflection
     */
    public ResultSetWritable() {
	
	this.results = null;
	this.name = "empty instance";
	this.info = "this is an empty instance of ResultSetWritable";
	
    }

    /**
     * Create a new resultset.
     *
     * @param DocumentWritable[] results       the ordered documents
     * @param String name              the server name
     * @param String info              any additional information
     */
    public ResultSetWritable(DocumentWritable[] results, String name, String info) {

	this.results = results;
	this.name = name;
	this.info = info;

    }


    /**
     * @param DocumentWritable[]
     */
    public void set(DocumentWritable[] results) {

	this.results = results;

    }

    /**
     * @return DocumentWritable[]
     */
    public DocumentWritable[] get() {

	return(results);

    }

    /**
     * @param DocumentWritable[] setResults
     */
    public void addResults(DocumentWritable[] newSet) {

	//First, add them to the OrderedList
	for(int x = 0; x < newSet.length; x++) {
	    
	    ranked.addRanked(newSet[x]);
	    
	}

	if(results == null) {

	    //our first set!
	    results = newSet;

	} else {

	    results = null;
	    results = new DocumentWritable[ranked.size()];
	    
	    for(int i = 0; i < ranked.size(); i++) {

		results[i] = (DocumentWritable)ranked.get(i);

	    }

	}


    }


    /**
     * @return DocumentWritable[] results
     */
    public DocumentWritable[] getResults() {

	return(results);

    }

    /**
     * @return String name
     */
    public String getName() {

	return(name);

    }

    /**
     * @return String info
     */
    public String getInfo() {

	return(info);

    }

    /** 
     * @param String name
     */
    public void setName(String name) {

	this.name = name;

    }

    /**
     * @param String info
     */
    public void setInfo(String info) {

	this.info = info;

    }

    /**
     * @param DocumentWritable[] results
     */
    public void setResults(DocumentWritable[] results) {

	this.results = results;

    }


}
