/**
 * A class that reads the records provided by the generated service file
 * and makes them into useable InputSplits.
 *
 * <p>
 * Last Updated: 5 August 2008
 * <p>
 *
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch;

import java.io.IOException;
import org.apache.hadoop.mapred.RecordReader;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileSplit;
import java.io.StringReader;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.BufferedInputStream;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.InputSource;

public class ServiceRecordReader implements RecordReader<Text, ServiceWritable> {
    //positions in the stream
    private long start;
    private long pos;
    private long end;
    private int newsize;

    /** Object that reads in Services via XML.*/
    private ServiceReader in = null;

    /**
     * A class that provides the reader from an input stream.
     */
    public static class ServiceReader {

	private static final int DEFAULT_BUFFER_SIZE = 64 * 1024;
	private int bufferSize = DEFAULT_BUFFER_SIZE;
	private InputStream in;
	private byte[] buffer;
	// the number of bytes of real data in the buffer
	private int bufferLength = 0;
	// the current position in the buffer
	private int bufferPosn = 0;
	private String xmltopper = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";


	/**
	 * @param InputStream 
	 */
	public ServiceReader(InputStream is) {

	    this.in = is;
	    this.buffer = new byte[this.bufferSize];

	}

	/**
	 * Fill the buffer with more data.
	 * @return was there more data?
	 * @throws IOException
	 */
	boolean backfill() throws IOException {

	    bufferPosn = 0;
	    bufferLength = in.read(buffer);
	    return bufferLength > 0;

	}
    


	/**
	 * Closes all objects
	 */
	public void close() throws java.io.IOException {

	    in.close();

	}

	/**
	 * Reads in a line from the input
	 *
	 * @param String
	 * @return int size
	 */
	public int readLine(Text str) throws java.io.IOException {

	    boolean hadFinalNewline = false;
	    boolean hadFinalReturn = false;
	    boolean hitEndOfFile = false;
	    int startPosn = bufferPosn;

	    outerLoop: while (true) {
		if (bufferPosn >= bufferLength) {
		    if (!backfill()) {
			hitEndOfFile = true;
			break;
		    }
		}
		startPosn = bufferPosn;
		for(; bufferPosn < bufferLength; ++bufferPosn) {
		    switch (buffer[bufferPosn]) {
		    case '\n':
			hadFinalNewline = true;
			bufferPosn += 1;
			break outerLoop;
		    case '\r':
			if (hadFinalReturn) {
			    // leave this \n in the stream, so we'll get it next time
			    break outerLoop;
			}
			hadFinalReturn = true;
			break;
		    default:
			if (hadFinalReturn) {
			    break outerLoop;
			}
		    }        
		}
		int length = bufferPosn - startPosn - (hadFinalReturn ? 1 : 0);
		if (length >= 0) {
		    str.append(buffer, startPosn, length);
		}
	    }
	    int newlineLength = (hadFinalNewline ? 1 : 0) + (hadFinalReturn ? 1 : 0);
	    if (!hitEndOfFile) {
		int length = bufferPosn - startPosn - newlineLength;
		if (length > 0) {
		    str.append(buffer, startPosn, length);
		}
	    }
	 
	    return str.getLength() + newlineLength;
	}

	/**
	 * Reads in the values for this record into a Service object.
	 *
	 * @param ServiceWritable
	 * @return int size
	 */
	public int readService(ServiceWritable sw) {

	    try { 

		int size = 0;
		String xmlstring = xmltopper;
		
		//We know that each <service> object is 5 lines
		for(int i = 0; i < 5; i++) {
		    
		    Text text = new Text();
		    size += readLine(text);

		    String string = text.toString();

		    if(string == null || string.equals("")) {
			
			return(-1);

		    } else {
			
			xmlstring += string;

		    }

		}

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = factory.newDocumentBuilder();
		InputSource instream = new InputSource();
		instream.setCharacterStream(new StringReader(xmlstring));
		org.w3c.dom.Document document = db.parse(instream);    
		
		NodeList uNL = document.getElementsByTagName("url");
		NodeList nNL = document.getElementsByTagName("name");
		NodeList cNL = document.getElementsByTagName("clientclass");
		
		Node urlNode = uNL.item(0);
		Node nameNode = nNL.item(0);
		Node clientNode = cNL.item(0);
		
		org.w3c.dom.Text urlText = (org.w3c.dom.Text)urlNode.getFirstChild();
		org.w3c.dom.Text nameText = (org.w3c.dom.Text)nameNode.getFirstChild();
		org.w3c.dom.Text clientText = (org.w3c.dom.Text)clientNode.getFirstChild();
		
		String URL = urlText.getNodeValue().trim();
		String name = nameText.getNodeValue().trim();
		String client = clientText.getNodeValue().trim();
		
		sw.setURL(URL);
		sw.setName(name);
		sw.setClientClass(client);
		
		return(size);

	    } catch(java.io.IOException ioe) {

		ioe.printStackTrace();
		return(0);

	    } catch(Exception e) {

		e.printStackTrace();
		return(0);

	    }

	}

    } // end in-class declaration
	 
    /**
     * @param Configuration
     * @param FileSplit
     * @param String
     * @throws IOException
     */
    public ServiceRecordReader(Configuration job, FileSplit split) 
	throws java.io.IOException {

	try { 

	    //get the positions in the stream
	    start = split.getStart();
	    end = split.getLength();
	    this.pos = start;

	    //File paths and such 
	    final Path file = split.getPath();
	    FileSystem fs = file.getFileSystem(job);
	    FSDataInputStream fileIn = fs.open(split.getPath());
	    in = new ServiceReader(fileIn);	     	    

	} catch(java.lang.NullPointerException npe) {

	    System.out.println("ServiceRecordReader@ " + npe.toString());
	    npe.printStackTrace();

	}

    }
    /**
     * @return long
     */
    public long getPos() {

	return(pos);

    }


    /**
     * Returns number of bytes used by object. (Between 0 and 1 should be returned)
     *
     * @return float
     * @throws java.io.IOException
     */
    public float getProgress() throws java.io.IOException {

	if (start == end) {

	    return 0.0f;

	} else {

	    return Math.min(1.0f, (pos - start) / (float)(end - start));

	}

    }

    /**
     * Close this split to future operations.
     */
    public void close() {

	try {

	    if(in != null) {

		in.close();

	    }

	} catch(java.io.IOException e) {
	    
	    //error catching
	    System.out.println("@ServiceRecordReader: " + e.toString());
	}

    }

    /**
     * Create a ServiceWritable object to be use as the value for this key, value pair
     *
     * @return ServiceWritable
     */
    public ServiceWritable createValue()  {

	ServiceWritable sw = new ServiceWritable();

	newsize = in.readService(sw);

	return(sw);

    }

    /** 
     * Return the new size.
     * 
     * @return int
     */
    public int getSize() {

	return(newsize);

    }
    

    /**
     * Generate an object for this to be used as a key.
     *
     * @return Text
     */
    public Text createKey() {

	return(new Text(Multisearch.getQuery()));

    }

    /**
     * Reads the next service into the object
     *
     * @param Text key
     * @param ServiceWritable service
     * @return boolean
     * @throws IOException
     */
    public synchronized boolean next(Text key, ServiceWritable service) 
				    throws IOException {

	    if(pos >= end) {

		return(false);

	    }

	    key = createKey();
	    // service = createValue();
	    //int size = in.readService(service);
	    int size = getSize();

	    if(size > 0) {

		pos+= size;
		return(true);

	    }

	    return(false);


	}

}
