/**
 * An {@link OutputFormat} that writes plain text files from
 * ResultWritable objects. This format is traditional TREC-style formating.
 *<p>
 * Last Update: 31 July 2008
 * <p>
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.RecordWriter;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.util.*;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.mapred.FileOutputFormat;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;

public class TRECOutputFormat extends FileOutputFormat<Text, ResultSetWritable> {

    //Declaring a class within this class, taken mostly from Hadoop
    protected static class ResultSetRecordWriter implements RecordWriter<Text, ResultSetWritable> {

	/** The object that writes to the file.*/
	DataOutputStream dos;
	OutputStreamWriter osw;
	BufferedWriter out;

	/* Taken from hadoop code. */
	private static final String utf8 = "UTF-8";

	
	/**
	 * Create a new writer with given output stream.
	 *
	 * @param DataOutputStream
	 */
	public ResultSetRecordWriter(DataOutputStream dos) {
	    
	    this.dos = dos;
	    osw = new OutputStreamWriter(dos);
	    out = new BufferedWriter(osw);
	    
	}

	/**
	 * Write an object to the byte stream
	 * Taken from Hadoop's LineRecordWriter class
	 *
	 * @param Object
	 */
	private void writeObject(Object o) throws IOException {

	    out.write(o.toString());
		
	}

	/**
	 * Close objects
	 */
	public void close(Reporter reporter) {
	    
	    try {
		
		out.close();
		osw.close();
		dos.close();

	    } catch(IOException ioe) {
		
		reporter.setStatus(ioe.toString());
		
	    }

	}

	/**
	 * Write this value to the stream
	 *
	 * @param Text
	 * @param ResultSet
	 * @throws IOException
	 */
	public synchronized void write(Text text, ResultSetWritable set) throws IOException {
			    
		if(set != null) {

		    DocumentWritable[] docs = set.getResults();

		    if(docs != null) {
		    
			for(int x = 0; x < docs.length; x++) {
			    
			    //TREC output style: # (of query) q0 docid rank score label
			    String writeMe = new String(Multisearch.getQueryNumber() + 
							//docs[x].getQueryNumber() +
							" q0 " + 
							docs[x].getRank() + " " +
							docs[x].getDocID() + " " +
							docs[x].getScore() + " " + 
							docs[x].getLabel() +
							 "\n");

			    writeObject(writeMe);
			
			}

		    } 
		    
		}
		
	    }

    }//end ResultSetWriter-declaration

    /** 
     * @param FileSystem
     * @param JobConf
     * @param String
     * @param Progressable
     */
    public ResultSetRecordWriter getRecordWriter(FileSystem ignored, JobConf job, String name, 
						 Progressable progress) throws IOException {


	Path dir = getWorkOutputPath(job);
	FileSystem fs = dir.getFileSystem(job);
	
	if(!fs.exists(dir)) {

	    throw new IOException("Output directory does not exist.");

	} 

	Path file = new Path(dir, name);
	boolean isCompressed = getCompressOutput(job);
	
	if(!isCompressed) {

	    FSDataOutputStream fileOut = fs.create(file, progress);
	    return new ResultSetRecordWriter(fileOut);

	} else {

	    throw new java.io.IOException("Multisearch does not process compressed files.");

	}

    }

}
