/**
 * Extention of the OGSA-DAI Activity, which allows for
 * new creators of Multisearch to add new Client Toolkit
 * Activities for new resources. This allows for flexible
 * perform documents.
 * <p>
 * This requires various things that Multisearch uses to
 * search different types of data. New Activities should
 * extend this.
 * <p>
 * DATE CREATED: 17 July 2007
 * LAST UPDATED: 25 July 2007
 *
 * @author OGSA-DAI, Kylie McCormick
 */
package edu.arsc.multisearch.client;

import uk.org.ogsadai.client.toolkit.activity.*;
import uk.org.ogsadai.client.toolkit.activity.NoActivityOutputException;
import uk.org.ogsadai.client.toolkit.exception.DataFormatException;

public class LuceneSearchToolkit extends SearchToolkit
{

    /** Index of search data output */
    private static final int DATA_OUTPUT_INDEX = 0;

    /** Index of search status output */
    private static final int STATUS_OUTPUT_INDEX = 1;

    /**
     * Constructs an activity with no inputs or outputs.
     */
    public LuceneSearchToolkit()
    {
        super();
	addOutput("indexedFileSearchOutput");
	addOutput("indexedFileSearchStatus");
    }
    
    /**
     * This allows forClass().getInstance() structure
     */
    public LuceneSearchToolkit getInstance() {

	return(new LuceneSearchToolkit());
    }
    

    
    /**
     * TAKEN FROM OGSA-DAI: IndexedFileSearch Activity
     *
     * Gets the activity's search status output.
     * 
     * @return the search status.
     */    
    public ActivityOutput getSearchStatus()
    {
        ActivityOutput queryResultOutput = getOutputs()[STATUS_OUTPUT_INDEX];
        return queryResultOutput;
    }

    /**
     * TAKEN FROM OGSA-DAIL IndexedFileSearch
     * Modified.
     *
     * Gets the number of results as recorded in the search status
     * output. 
     * 
     * @return number of search results.
     * @throws DataFormatException
     *     if the XML output data for the activity's output cannot be
     *     parsed. 
     * @throws NoActivityOutputException
     *     if there is no output data or the data has not yet been
     *     processed to this output or the activity that generates
     *     this output has not yet been executed.
     */    
    public long getResultCount() 
        throws NoActivityOutputException, 
               DataFormatException 
    {
        long count = 0;
        ActivityOutput output = getOutputs()[STATUS_OUTPUT_INDEX];
        String status = output.getData();
        String startTag = "<searchResult resultCount=\"";
        if (! status.startsWith(startTag)) {

            throw new DataFormatException(startTag, status);
        }
        String endTag = "\">";
        int index = status.indexOf(endTag, startTag.length());
        String countStr = status.substring(startTag.length(), index);
        try 
	    {
		count = Long.parseLong(countStr);
	    }
        catch (NumberFormatException e) 
	    {
		throw new DataFormatException("java.lang.Integer", status, e);
	    }        

	return(count);


    }


    /* (non-Javadoc) TAKEN FROM OGSADAI: IndexedFileSearch
     * @see uk.org.ogsadai.client.toolkit.Activity#generateXML()
     */
    protected String generateXML() 
    {
        StringBuffer buf = new StringBuffer();
        buf.append("<luceneSearchToolkit name=\"");
        buf.append(getName());
        buf.append("\">\n");
	buf.append("<min>" + min + "</min>\n");
	buf.append("<max>" + max + "</max>\n");
        buf.append("<query>");
	//System.out.println(query);
        buf.append(replaceSpecialCharacters(query));
        buf.append("</query>\n");
        if (index != null) 
	    {
		buf.append("<index>");
		buf.append(replaceSpecialCharacters(index));
		buf.append("</index>\n");
	    }
        buf.append("<output name=\"");
        buf.append(getOutputs()[DATA_OUTPUT_INDEX].getName());
        buf.append("\"/>\n");
        buf.append("<status name=\"");
        buf.append(getOutputs()[STATUS_OUTPUT_INDEX].getName());
        buf.append("\"/>\n");
        buf.append("</luceneSearchToolkit>");
        return buf.toString();
    }


}
