/**
 * An object to represent an OGSA-DAI resource with information
 * Multisearch needs for variability.
 * <p>
 * DATE CREATED: 16 July 2007
 * LAST UPDATED: 7 August 2007
 *
 * @author Kylie McCormick
 */
package edu.arsc.multisearch;

import uk.org.ogsadai.common.msgs.DAILogger;
import java.util.LinkedList;
import uk.org.ogsadai.client.toolkit.properties.Property;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import javax.xml.namespace.QName;

public class ResourceBlock {

    /** OGSA-DAI Logger object */
    private static final DAILogger LOG = DAILogger.getLogger(ResourceBlock.class);

    /** OGSA-DAI ID for Resource */
    private String id;

    /** Toolkit Implementation class name */
    private String clientToolkitName;

    /** Is this resource secure? */
    private boolean secure;

    /** The resource index, if needed. */
    private String index;

    /** The String of output generated from this resource. */
    private String output;

    /** The QNames associated with the metadata for this resource. */
    private QName[] qNames;

    /** The MetaData aquried when checking this resource for validity. */
    private Property[] metadata;

    /** The MetaData aquired in String form. **/
    private String[] metaData;

    /** The MetaData Names, parallel to String[] metaData. */
    private String[] metaDataNames;

    /** The LinkedList of documents generated from this resource. */
    private LinkedList docs = null;


    /**
     * @param String id                   the OGSA-DAI id of this resource
     * @param String clientToolkitName    implementation class for this resource
     * @param String index                the index to be searched
     * @param boolean secure              if this is a secure resource
     * @param QName[] qNames              all of the QNames associated with metaData
     * @return ResourceBlock
     */
    public ResourceBlock(String id, String clientToolkitName, String index, boolean secure, QName[] qNames) {

	this.id = id;
	this.clientToolkitName = clientToolkitName;
	this.secure = secure;
	this.index = index;
	this.qNames = qNames;

	if(LOG.isDebugEnabled()) {

	    LOG.debug("New Resource block " + id + " created!");

	}

    }


    /**
     * @return String id     the OGSA-DAI id of this resources
     */
    public String getID() {

	return(id);

    }

    /** 
     * @return String clientToolkitName   the classname for the toolkit 
     */
    public String getClientToolkitName() {

	return(clientToolkitName);
    }


    /**
     * @return boolean    if this is secure, true
     */
    public boolean isSecure() {

	return(secure);

    }

    /**
     * @param String output     the output (CDATA) of this resource
     */
    public void setOutput(String output) {

	this.output = output;

    }

    /**
     * @return String index     the index to search (may be null)
     */
    public String getIndex() {

	return(index);

    }

    /**
     * @return LinkedList docs    the documents from this resource
     */
    public LinkedList getDocs() {
	
	return(docs);

    }

    /** 
     * @param LinkedList docs     the output documents for this set
     */
    public void setDocs(LinkedList docs) {

	this.docs = docs;

    }

    /**
     * @param Property[]     meta data
     */
    public void setMetaData(Property[] properties) {

	this.metadata = properties;

	if(metadata != null) {

	    metaData = new String[properties.length];
	    metaDataNames = new String[properties.length];

	    for(int i = 0; i < properties.length; i++) {

		Element elt = (Element)properties[i].getValue();
		metaDataNames[i] = elt.getTagName();
		metaData[i] = elt.getAttribute("value");
	    }

	} else {

	    metaData = null;

	}

    }

    /**
     * @return Property
     */
    public Property[] getMetaData() {

	return(metadata);

    }

    /**
     * @return String[]
     */
    public String[] getStringMetaData() {

	if(metadata == null) {

	    return(null);

	} else {

	    return(metaData);

	}

    }

    /**
     * @return QNames[]
     */
    public QName[] getQNames() {

	return(qNames);

    }

    /**
     * @return String[]
     */
    public String[] getMetaDataNames() {

	return(metaDataNames);

    }


    /**
     * @return String[]
     */
    public String getMetaDataByName(String name) {


	String returnMeta = "";

	
	for(int i = 0; i < metaDataNames.length; i++) {

	    if(metaDataNames[i].equals(name)) {
		
		returnMeta = metaData[i];
		break;


	    }

	}


	return(returnMeta);


    }


}
