/**
 * A class that reads the records provided by the generated service file
 * and makes them into useable InputSplits.
 *
 * <p>
 * Last Updated: 4 August 2008
 * <p>
 *
 * @author Kylie McCormick
 * @version 3.0
 */
package edu.arsc.multisearch;

import java.io.IOException;
import org.apache.hadoop.mapred.RecordReader;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileSplit;
import java.io.StringReader;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.BufferedInputStream;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.InputSource;

public class ServiceRecordReader implements RecordReader<Text, ServiceWritable> {

    /** Object that reads in Services via XML.*/
    private ServiceReader in = null;

    /**
     * A class that provides the reader from an input stream.
     */
    public static class ServiceReader {

	//should upgrade to non-deprecated variation
	private DataInputStream in;
	private BufferedInputStream bis;
	private InputStream inStream;
	private String topper = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
	private int size;

	/** 
	 * Create a new ServiceReader from the given stream
	 *
	 * @param InputStream in
	 * @throws java.io.IOException
	 */
	ServiceReader(InputStream inStream) throws java.io.IOException {

	    this.inStream = inStream;
	    this.bis = new BufferedInputStream(inStream);
	    this.in = new DataInputStream(bis);
	    this.size = in.available();
	}

	/**
	 * @return int size
	 */
	public int getSize() {

	    return(size);

	}

	/**
	 * Closes the underlying stream.
	 *
	 * @throws IOException
	 */
	public void close() throws IOException {

	    in.close();
	    bis.close();
	    inStream.close();
	    
	}

	/**
	 * Returns the current position in the input.
	 *
	 * @return int
	 */
	public int getAvailable() throws java.io.IOException {

	    in.mark(1000);

	    if(in.readLine() == null) {
		
		return(-1);
	    }

	    in.reset();

	    return(in.available());

	}

	/**
	 * Reads in an XML chunk that defines a service. &lt;service&gt;&lt;/service&gt;
	 *
	 * @param ServiceWritable to update
	 * @throws java.io.IOException, javax.xml.parsers.ParserConfigurtionException, org.xml.sax.SAXException
	 */	 
	public void readService(ServiceWritable sw) 
	    throws java.io.IOException, javax.xml.parsers.ParserConfigurationException, org.xml.sax.SAXException{

	    String xmlstring = topper;
	    boolean complete = false;
	    String input = in.readLine();
	    xmlstring += input;

	    if(!complete) {

		while(input != null && !input.equals("</service>")) {
		    
		    input = in.readLine();
		    xmlstring += input;
		
		} //end-while-not-xml-service-end
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = factory.newDocumentBuilder();
		InputSource instream = new InputSource();
		instream.setCharacterStream(new StringReader(xmlstring));
		org.w3c.dom.Document document = db.parse(instream);	    
		
		NodeList uNL = document.getElementsByTagName("url");
		NodeList nNL = document.getElementsByTagName("name");
		NodeList cNL = document.getElementsByTagName("clientclass");
		
		Node urlNode = uNL.item(0);
		Node nameNode = nNL.item(0);
		Node clientNode = cNL.item(0);
		
		org.w3c.dom.Text urlText = (org.w3c.dom.Text)urlNode.getFirstChild();
		org.w3c.dom.Text nameText = (org.w3c.dom.Text)nameNode.getFirstChild();
		org.w3c.dom.Text clientText = (org.w3c.dom.Text)clientNode.getFirstChild();
		
		String URL = urlText.getNodeValue().trim();
		String name = nameText.getNodeValue().trim();
		String client = clientText.getNodeValue().trim();
		
		sw.setURL(URL);
		sw.setName(name);
		sw.setClientClass(client);
	   	complete = true;    
	    } 
	    
	}

    } //end-inner-class-declaration
	 
    /**
     * @param Configuration
     * @param FileSplit
     * @param String
     * @throws IOException
     */
    public ServiceRecordReader(Configuration job, FileSplit split) 
	throws java.io.IOException {

	try { 

	    final Path file = split.getPath();
	    FileSystem fs = file.getFileSystem(job);
	    FSDataInputStream fileIn = fs.open(split.getPath());
	    in = new ServiceReader(fileIn);	     	    

	} catch(java.lang.NullPointerException npe) {

	    System.out.println("ServiceRecordReader@ " + npe.toString());
	    npe.printStackTrace();

	}

    }


    /**
     * Returns number of bytes used by object. (Between 0 and 1 should be returned)
     *
     * @return float
     * @throws java.io.IOException
     */
    public float getProgress() throws java.io.IOException {

	if(in.getAvailable() < 0) {

	    return(0);

	} else {
	    
	    float val = (float)in.getAvailable() / (float)in.getSize();

	    return(val);

	}

    }

    /**
     * Close this split to future operations.
     */
    public void close() {

	try {

	    if(in != null) {

		in.close();

	    }

	} catch(java.io.IOException e) {

	    //error catching
	    System.out.println("@ServiceRecordReader: " + e.toString());
	}

    }

    /**
     * Returns current position in the input.
     *
     * @return long
     * @throws java.io.IOException
     */
    public long getPos() throws java.io.IOException {

	//long value = (long)((long)(in.getAvailable()) / (long)in.getValue());

	return((long)in.getAvailable());

    }

    /**
     * Create a ServiceWritable object to be use as the value for this key, value pair
     *
     * @return ServiceWritable
     */
    public ServiceWritable createValue()  {

	try { 

	    ServiceWritable sw = new ServiceWritable();
	    in.readService(sw);
	    return(sw);

	} catch(Exception e) {

	    e.printStackTrace();
	    return(null);

	}

    }

    /**
     * Generate an object for this to be used as a key.
     *
     * @return Text
     */
    public Text createKey() {

	String textKey = Multisearch.getQuery();
	Text text = new Text(textKey);
	return(text);

    }

    /**
     * Reads the next service into the object
     *
     * @param Text key
     * @param ServiceWritable service
     * @return boolean
     * @throws IOException
     */
    public synchronized boolean next(Text key, ServiceWritable service) 
				    throws IOException {

	    //check for end of stream
	    if(in.getAvailable() < 0) {

		return(false);
		
	    } else {

		try {

		    //generate the service associated with this
		    service = createValue();
		    
		    //generate the key associated with this
		    key = createKey();	
		
		    System.out.println("New service " + service.getURL());
		    
		    return(true);

		} catch(Exception e) {

		    e.printStackTrace();
		    return(false);
		}

	    }	    

    }

}
