/**
 * In order to have the searches running simultaneously, we need
 * threads that can do all teh operations for us. Search Thread loads
 * the connection to the Data Service, runs the query, and packages
 * it into Documents. Then, it saves the output to the ResourceBlock
 * to be used later as needed.
 * <p>
 * DATE CREATED: 19 July 2007
 * LAST UPDATED: 11 June 2008
 *
 * @author Kylie McCormick
 */
package edu.arsc.multisearch;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.LinkedList;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.ResourceID;
import uk.org.ogsadai.client.toolkit.service.DataService;
import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.Response;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.ActivityOutput;
import uk.org.ogsadai.client.toolkit.activity.indexedfiles.IndexedFileSearch;
import uk.org.ogsadai.client.toolkit.activity.indexedfiles.ReadFile;
import uk.org.ogsadai.common.msgs.DAILogger;
import edu.arsc.multisearch.merge.*;
import edu.arsc.multisearch.exception.*;
import uk.org.ogsadai.client.toolkit.activity.Activity;
import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.client.toolkit.ResourceID;
import java.lang.Thread;

public class SearchThread extends Thread {

    /** Print Writer object for Java Servlet. */
    public static PrintWriter out;
    
    /** OGSA-DAI Logger object. */
    private static final DAILogger LOG = DAILogger.getLogger(SearchThread.class);

    /** Minimum number of results*/
    private int max;

    /** Maximum number of results*/
    private int min;

    /** Server object, which has the service's information */
    private Server server;

    /** The resource we're using for this search thread*/
    private ResourceBlock resource;

    /** The query over this thread */
    private String query;

    /** The research toolkit */
    SearchToolkit resourceToolkit = null;

    /** 
     * Creates a new SearchThread to be run.
     *
     * @param Server           the object representing the service
     * @param ResourceBlock    the object pointing to the resource we want
     * @param int              the minimun number of documents to be returned
     * @param int              the maximum number of documents to be returned
     * @param String           the query
     * @throws NoToolkitClassException   when the toolkit cannot be found     
     * @throws BadToolkitClassException  if the toolkit does not accept the right params
     */
    public SearchThread(Server server, ResourceBlock resource, int min, int max, String query) throws NoToolkitClassException, BadToolkitClassException {

	//grab all the values
	this.server = server;
	this.resource = resource;
	this.min = min;
	this.max = max;
	//this.query = query.toLowerCase();
	this.query = query;

	//initialize the values we intend to use
        try {

            Class toolkit = Class.forName(resource.getClientToolkitName());
            resourceToolkit = (SearchToolkit)toolkit.newInstance();

        } catch(Exception e) {

	    LOG.error(e, true);
	    throw new NoToolkitClassException("SearchThread@SearchThread()");
	    
        }

	//setting up the resource toolkit
	try {
	    
            resourceToolkit.setMin(min);
            resourceToolkit.setMax(max);
            resourceToolkit.setQuery(query);
            resourceToolkit.setIndex(resource.getIndex());

	} catch(Exception e) {

	    LOG.error(e, true);
	    throw new BadToolkitClassException("SearchThread@SearchThread()");
	}

    }

    /**
     * The thread opens up a connection to the given Data Service,
     * creates an activity, attaches its Toolkit, and sents a perform
     * document. When the results are back, it connects them to the
     * ResourceBlock.
     */
    public void run() {

	try {

	    //make the service connection
	    String handle = server.getURI();
	    handle = handle.trim();
	    String rID = resource.getID();
	    rID = rID.trim();

	    //open data services, and full steam ahead!
	    DataService service = GenericServiceFetcher.getInstance().getDataService(handle, rID);  
	    ActivityRequest request = new ActivityRequest();
	    request.add(resourceToolkit);
	    Response response = service.perform(request);

	    //Get the data in a usable form
	    ActivityOutput outputData = resourceToolkit.getSearchStatus();
	    String cData = outputData.getData();
	    	    
	    //Generate A Linked List
	    LinkedList docList = makeDocs(cData, server.getName(), resource.getID());

	    //save it to the resource, eh?
	    resource.setDocs(docList);

        } catch(DAIException e) {

	    LOG.error(e, true);
	    
	} catch(Exception e) {
	    
	    LOG.error(e, false);

	}

	if(LOG.isDebugEnabled()) {

	    LOG.debug("SearchThread@run() has completed.");

	}
      

    }

    /**
     * Turns XML responses into Document Objects that are
     * returned in an Array.
     *
     * @param String     the XML CDATA response string
     * @param String     the DataResourceName
     * @return           of Document objects
     * @throws NoDocumentInXMLException   when the documents are not represented properly in the given XML file
     */
    private LinkedList makeDocs(String rS, String name, String rName) 
	throws NoDocumentInXMLException {

	LinkedList responseList = new LinkedList();
       	File xmlfile = makeXMLFile(name, rName);

	if(xmlfile != null) {

	    try {

		java.io.FileOutputStream fos = new java.io.FileOutputStream(xmlfile);
		java.io.PrintStream printXML = new java.io.PrintStream(fos);

		printXML.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		printXML.println(rS);
	    
		//now we can treat it like an XML document
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		org.w3c.dom.Document response = factory.newDocumentBuilder().parse(xmlfile); 

		//processing...
		Element element = response.getDocumentElement();
		NodeList nodes = element.getElementsByTagName("document");
	
		if(nodes != null && nodes.getLength() > 0) {
		    
		    for(int d = 0; d < nodes.getLength(); d++) {

			//grab a node
			Element node = (Element)nodes.item(d);

			//change the string numbers into values
			String score = node.getAttribute("score");
			String rank = node.getAttribute("rank");
			String filePath = server.getBaseURI() + node.getAttribute("filename");
			String title = node.getAttribute("title");

			if(title == null || title.equals("")) {
			
			    title = "Untitled";
			}

			//int for rank
			java.lang.Integer integerX = new java.lang.Integer(rank);
			int rankIt = integerX.intValue();

			//double for score
			java.lang.Double doubleX = new java.lang.Double(score);
			double scoreIt = doubleX.doubleValue();

			Document newDoc = new Document(title, node.getAttribute("id"), rankIt, scoreIt, name, filePath);
		    
			responseList.add(newDoc);
		    }
		
		} else {
		
		    /*
		    String errorString = "SearchThread@makeDocs() " + rName;
		    LOG.error(new NoDocumentInXMLException(errorString));
		    */
		}
	    
		//clean up XML file
		xmlfile.delete();
	    
	    } catch(java.io.IOException e) {

		LOG.debug("The file should have been created by the program. It could be an overwrite or another problem.");
		LOG.error(e, false);

	    } catch(javax.xml.parsers.ParserConfigurationException e) {

		LOG.error(e, true);

	    } catch(org.xml.sax.SAXException e) {
		
		LOG.error(e, true);

	    }
	    
	}

	return(responseList);

    }

    /**
     * A function to create an XML file to be used by the parser.
     *
     * @param String        name of the server
     * @param String        name of the resource
     * @return File 
     */
    private File makeXMLFile(String server, String resource) {

	File xmlfile = null;
	
	try {

	    String uName = server.replace(" ", "");
	    uName = uName.replace(":", "");
	    uName = uName.trim();
	    
	    String urName = resource.replace(" ", "");
	    urName = urName.trim();
	    
	    //unique xmlfile name string
	    String xmlFileName = uName + "_" + urName + ".xml";

  	    xmlfile = File.createTempFile("output_", xmlFileName);

	} catch(java.io.IOException e) {

	    LOG.error(e, true);

	}

	return(xmlfile);

    }

}
