/**
 * A structure to represent servers/services with resources.
 * OGSA-DAI standards here have been met; these are for Multisearch
 * configuration needs, to represent servers and to hold the results 
 * for later use (eg-to call upon results in their initial order).
 * <p>
 * DATE CREATED: 16 July 2007
 * LAST UPDATED: 27 July 2007
 *
 * @author Kylie McComrick 
 */
package edu.arsc.multisearch;

import javax.xml.namespace.QName;
import uk.org.ogsadai.common.msgs.DAILogger;
import java.util.ArrayList;
import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.ResourceID;
import uk.org.ogsadai.client.toolkit.service.DataService;
import java.lang.Comparable;
import edu.arsc.multisearch.exception.*;

public class Server implements Comparable{


    /** OGSA-DAI logger object */
    private static final DAILogger LOG = DAILogger.getLogger(Server.class);

    /** Server/Service Name */
    private String serverName;

    /** Service URI */
    private String uri;

    /** All the Resources of this object, coupled with their type. */
    private ArrayList resources;

    /** The URI of the base of the service. */
    private String baseURI;

    /** The score of the server in case of a restriction algorithm. */
    private double score;

    /**
     * @param String serverName     Name of the server
     * @param String uri            location of the file
     * @param ArrayList resources   information on all the resources
     */
    public Server(String serverName, String uri, ArrayList resources)
	throws NoServiceException {

	this.serverName = serverName;
	this.uri = uri;
	this.resources = resources;
	this.score = 0.00;

	//Assumption: could have http://, but we want up to the third slash if so
	if(uri.startsWith("http://") || uri.startsWith("https://") || uri.startsWith("ftp://")) {
	    
	    int countSpace = uri.indexOf("://")+3;
	    int finalSpace = uri.indexOf("/", countSpace);
	    baseURI = uri.substring(0, finalSpace);
	}

	try {
	    
	    //this is to validate the service
	    DataService service = GenericServiceFetcher.getInstance().getDataService(uri, null);

	    //now the resources
	    ResourceID[] validIDs = service.getResourceIDs();
	    ArrayList validR = new ArrayList();
	    
	    //this is to validate the resources
	    for(int i = 0; i < resources.size(); i++) {

		ResourceBlock rb  = (ResourceBlock)resources.get(i);
		
		for(int j = 0; j < validIDs.length; j++) {

		    if(rb.getID().equals(validIDs[j].getName())) {
			
			service.setResourceID(validIDs[j]);
			
			try {
				
			    QName[] names = rb.getQNames();

			    if(names != null) {

				rb.setMetaData(service.getProperties(names));

			    }

			} catch(Exception e) {
				
			    rb.setMetaData(null);
			    LOG.error(e, true);
			}


			validR.add(rb);
      			break;
		    }

		} //end for-valid		

	    } //end-for

	    resources = validR;

	    } catch(Exception e) {

		LOG.error(e, true);
		e.printStackTrace();
		throw new NoServiceException("Server@Server() Resource Checker. ");

	    }

	if(LOG.isDebugEnabled()) {

	    LOG.debug("New Server has been created. Name: " + serverName);
	}

    }

    /**
     * @return String serverName
     * @return double score
     */
    public Server(String serverName, double score)
	throws NoServiceException {

	this.serverName = serverName;
	this.score = 0.00;


	String shortID = serverName.substring(10, serverName.length());


	/** Add the server's only resource. */
	ResourceBlock rbs = new ResourceBlock(shortID, "edu.arsc.multisearch.LuceneSearchToolkit", "DEFAULT", false, null);
	resources = new ArrayList();
	resources.add(rbs);

	//Assumption: could have http://, but we want up to the third slash if so
	String uriBase = "http://snowy.arsc.alaska.edu:8080/axis/services/ogsadai/Gov" + shortID;

	if(shortID.startsWith("10") || shortID.startsWith("12")  || shortID.startsWith("13")  || shortID.startsWith("14")  || shortID.startsWith("17")  || shortID.startsWith("18")) {
	    
	uriBase = "http://pileus.arsc.alaska.edu:8080/axis/services/ogsadai/Gov" + shortID;


	}

	this.uri = uriBase;


	try {
	    
	    //this is to validate the service
	    DataService service = GenericServiceFetcher.getInstance().getDataService(uri, null);

	    //now the resources
	    ResourceID[] validIDs = service.getResourceIDs();
	    ArrayList validR = new ArrayList();
	    
	    //this is to validate the resources
	    for(int i = 0; i < resources.size(); i++) {

		ResourceBlock rb  = (ResourceBlock)resources.get(i);
		
		for(int j = 0; j < validIDs.length; j++) {

		    if(rb.getID().equals(validIDs[j].getName())) {
			
			service.setResourceID(validIDs[j]);
			
			try {
				
			    QName[] names = rb.getQNames();

			    if(names != null) {

				rb.setMetaData(service.getProperties(names));

			    }

			} catch(Exception e) {
				
			    rb.setMetaData(null);
			    LOG.error(e, true);
			}


			validR.add(rb);
      			break;
		    }

		} //end for-valid		

	    } //end-for

	    resources = validR;

	    } catch(Exception e) {

		LOG.error(e, true);
		//e.printStackTrace();
		throw new NoServiceException("Server@Server() Resource Checker. ");

	    }

	if(LOG.isDebugEnabled()) {

	    LOG.debug("New Server has been created. Name: " + serverName);
	}

    }

    /**
     * @return ArrayList resources   all the resources of this service
     */
    public ArrayList getResources() {

	return(resources);

    }

    /**
     * @return String uri   the location of the service
     */
    public String getURI() {
	
	return(uri);
    }

    /**
     * @return String serverName   the Name of this server
     */
    public String getName() {

	return(serverName);

    }


    /**
     * @return int   number of valid resources for this server
     */
    public int getNumberOfResources() {

	return(resources.size());

    }

    /**
     * @param int                  the position of the resource
     * @return ResourceBlock       the resource at the given position or NULL
     */
    public ResourceBlock getResource(int x) {

	if(x < resources.size()) {

	    return((ResourceBlock)(resources.get(x)));

	} else {

	    return(null);

	}
    } 

    /**
     * @return String baseURI
     */
    public String getBaseURI() {

	return(baseURI);

    }

    /**
     * Compare servers via score for restriction algorithms
     * <p>                                                                                 
     * -1   If Object o is LESS THAN this object                                        
     * 0    If Object o is EQUAL TO this object                                         
     * 1    If Object o is GREATER THAN this object                                      
     * <p>
     * @throws ClassCastException     when not Object o not comparable to Server
     * @return int        -1, 0, or 1                                                
     */
    public int compareTo(Object o) {

        try {

            Server s = (Server)o;

            if(s.getScore() > this.getScore()) {

		return(1);

	    } else if(s.getScore() < this.getScore()) {

		return(-1);

            } else {

                return(0);
            }

        } catch(ClassCastException e) {

            LOG.error(new ClassCastException("Server@compareTo() - Cannot cast object."));
	}

	return(0);

    }

    /**
     * @param double    the score as assessed by a restriction algorithm
     */
    public void setScore(double x) {

	this.score = x;

    }

    /**
     * @return double
     */
    public double getScore() {

	return(score);

    }

}
